﻿namespace Eco.Mods.TechTree
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using Eco.Gameplay.Blocks;
    using Eco.Gameplay.Components;
    using Eco.Gameplay.Components.Auth;
    using Eco.Gameplay.Components.VehicleModules;
    using Eco.Gameplay.DynamicValues;
    using Eco.Gameplay.Economy;
    using Eco.Gameplay.Housing;
    using Eco.Gameplay.Interactions;
    using Eco.Gameplay.Items;
    using Eco.Gameplay.Minimap;
    using Eco.Gameplay.Objects;
    using Eco.Gameplay.Players;
    using Eco.Gameplay.Property;
    using Eco.Gameplay.Skills;
    using Eco.Gameplay.Systems.TextLinks;
    using Eco.Gameplay.Pipes.LiquidComponents;
    using Eco.Gameplay.Pipes.Gases;
    using Eco.Gameplay.Systems.Tooltip;
    using Eco.Shared;
    using Eco.Shared.Math;
    using Eco.Shared.Networking;
    using Eco.Shared.Localization;
    using Eco.Shared.Serialization;
    using Eco.Shared.Utils;
    using Eco.Shared.View;
    using Eco.Shared.Items;
    using Eco.Gameplay.Pipes;
    using Eco.World.Blocks;

    [Serialized]
    //[RequireComponent(typeof(OnOffComponent))]
    [RequireComponent(typeof(PropertyAuthComponent))]
    //[RequireComponent(typeof(MinimapComponent))]
    //[RequireComponent(typeof(PowerGridComponent))]
    //[RequireComponent(typeof(PowerConsumptionComponent))]
    //[RequireComponent(typeof(HousingComponent))]
   //[RequireComponent(typeof(ElevatorComponent))]
    public partial class IndustrialElevatorObject : PhysicsWorldObject, INetObjectPriority
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Industrial Elevator"); } }

        public float Priority { get { return NetObjectPriority.Medium; } }

        [Serialized]
        private float cagePos;
        private float v;

        private IndustrialElevatorObject() { }

        //protected override void OnCreate()
        //{
        //    base.OnCreate();
        //    this.cagePos = this.Position.y - 0.5f;
        //}

        protected override void Initialize()
        {
            base.Initialize();
            this.cagePos = this.Position.y - 0.5f;
            this.SetAnimatedState("dir", 0f);
        }

        //public override void SendInitialState(BSONObject bsonObj, INetObjectViewer viewer)
        //{
        //    base.SendInitialState(bsonObj, viewer);
        //    this.SendUpdate(bsonObj, viewer);
        //    bsonObj["dir"] = this.GetAnimatedState<float>("dir");
        //}

        //public override void SendUpdate(BSONObject bsonObj, INetObjectViewer viewer)
        //{
        //    base.SendUpdate(bsonObj, viewer);
        //    bsonObj["cagePos"] = this.cagePos;
        //    bsonObj["v"] = this.v;
        //}

        //public override void ReceiveUpdate(BSONObject bsonObj)
        //{
        //    BSONValue cageUpdate;
        //    if (bsonObj.TryGetValue("cage", out cageUpdate))
        //    {
        //        this.cagePos = cageUpdate.ObjectValue["pos"].Vector3Value.y;
        //        bsonObj["time"] = cageUpdate.ObjectValue["time"].FloatValue;
        //        if (cageUpdate.ObjectValue.ContainsKey("v"))
        //            this.v = cageUpdate.ObjectValue["v"].Vector3Value.y;
        //    }
        //    base.ReceiveUpdate(bsonObj);

        //    if (bsonObj.ContainsKey("stop"))
        //    {
        //        this.v = 0f;
        //        this.AnimatedStates["dir"] = 0f;
        //    }
        //    this.SetDirty();
        //}

        public override InteractResult OnActInteract(InteractionContext context)
        {
            if (context.Parameters != null && context.Parameters.ContainsKey("Down"))
            {
                var physicsEntity = this.netEntity as NetPhysicsEntity;
                physicsEntity.SetPhysicsController(context.Player);
                this.AnimatedStates["dir"] = -1f;
            }
            else if (context.Parameters != null && context.Parameters.ContainsKey("Up"))
            {
                var physicsEntity = this.netEntity as NetPhysicsEntity;
                physicsEntity.SetPhysicsController(context.Player);
                this.AnimatedStates["dir"] = 1f;
            }
            else if (context.Parameters != null && context.Parameters.ContainsKey("Stop"))
            {
                var physicsEntity = this.netEntity as NetPhysicsEntity;
                physicsEntity.SetPhysicsController(context.Player);
                this.AnimatedStates["dir"] = 0f;
            }
            return base.OnActInteract(context);
        }

        public override void Destroy()
        {
            base.Destroy();
        }
        //private IndustrialElevatorObject() { }

        //protected override void Initialize()
        //{
        //    //this.GetComponent<ElevatorComponent>().Initialize();
        //    //base.Initialize();
        //    //var elevatorComponent = this.GetComponent<ElevatorComponent>();
        //    //elevatorComponent.Initialize();
        //}

    }

    [Serialized]
    [Weight(10000)]
    public partial class IndustrialElevatorItem : WorldObjectItem<IndustrialElevatorObject>
    {
        public override LocString DisplayName { get { return Localizer.DoStr("Industrial Elevator"); } }
        public override LocString DisplayDescription { get { return Localizer.DoStr("An Elevator for heavy duty."); } }

        static IndustrialElevatorItem()
        {

        }
    }

    [RequiresSkill(typeof(MechanicsSkill), 0)]
    public class IndustrialElevatorRecipe : Recipe
    {
        public IndustrialElevatorRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<IndustrialElevatorItem>(),
            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<PortableSteamEngineItem>(1),
                new CraftingElement<GearboxItem>(typeof(MechanicsSkill), 10, MechanicsSkill.MultiplicativeStrategy),
                new CraftingElement<CelluloseFiberItem>(typeof(MechanicsSkill), 20, MechanicsSkill.MultiplicativeStrategy),
                new CraftingElement<LumberItem>(typeof(MechanicsSkill), 30, MechanicsSkill.MultiplicativeStrategy)
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(IndustrialElevatorRecipe), Item.Get<IndustrialElevatorItem>().UILink(), 25, typeof(MechanicsSkill), typeof(MechanicsFocusedSpeedTalent));

            this.Initialize(Localizer.DoStr("Industrial Elevator"), typeof(IndustrialElevatorRecipe));
            CraftingComponent.AddRecipe(typeof(AssemblyLineObject), this);
        }
    }
}